Attribute VB_Name = "modSort"
'---------------------------------------------------------------------------------------
' Module    : modSort
' DateTime  : 06/01/2009 18:18
' Author    : Gary Conway
' Purpose   :
'
'
'       This module implements various sorting algorithms
'
'       BubbleSortNumbers( arrayname)
'       Quicksort( arrayname,lowbound,highbound)
'       SelectionSortNumbers( arrayname)
'       ShellSortNumbers(arrayname)
'
'
'
'---------------------------------------------------------------------------------------

Option Explicit

Public UseDelay As Boolean

Public Compares As Long
Public Swaps As Long
Dim Xleft As Long
Dim Hlen As Long

Public Seconds As Long
Public TimerInterval As Long

Public Const NumElements = 20
Public SortIndex As Long
Public TimeExpired As Boolean
Public CloseClick As Boolean

Public SortArray(0 To 19) As Long
Public Half As Long


Public Sub SetVals()
    On Error Resume Next
    Xleft = Main.Element(0).Left + Main.Element(0).Width + 10
    Hlen = 200
    Half = (Main.Element(0).Height / 2) - 40
End Sub


'   This is an optimized Bubble Sort, in that, it is
'   intelligent enough to realize that it doesnt need
'   to traverse the list completely each time. Since one
'   iteration moves the highest value to the end of the list.
'   It therefore, traverses to the point the last insert was
'   made on the last pass. As the algorithm traverses the list,
'   the smaller values "bubble" to the top of the list.


Public Sub BubbleSortNumbers(varray As Variant)
   
   Dim cnt1 As Long
   Dim cnt2 As Long
   Dim tmp As Long
   Dim counter As Long
   

For cnt1 = UBound(varray) To LBound(varray) Step -1

    If CloseClick Then Exit Sub

    For cnt2 = LBound(varray) + 1 To cnt1
        Compares = Compares + 1
        ShowSwaps
      If varray(cnt2 - 1) > varray(cnt2) Then
         tmp = varray(cnt2 - 1)
         varray(cnt2 - 1) = varray(cnt2)
         varray(cnt2) = tmp
        Swaps = Swaps + 1
        ShowSwaps
      End If

      DisplayLines cnt2, cnt2 - 1
      
    Next cnt2

Next cnt1

    ShowSwaps


End Sub




'   This is a recursive sort.
'   The QuickSort divides the list in half, then quarters etc. and
'   "squeezes" toward the center of the division (pivot) comparing
'   values above and below the pivot and swapping as necessary.


''Public Type vrs
''    Name As String          ' name of the variable as listed in the source file
''    LineNumber As Long      ' line number where the variable is defined
''    FileName As String      ' filename where the variable is defined
''    VarType As String       ' variable type
''    References() As String  ' the following two arrays work in conjunction, references holds filenames
''    LineNums() As Long      ' where the variable is referenced and linenums holds the line number
''End Type
''
''Public Variables() As vrs


Public Sub QuickSort(varray As Variant, ByVal inLow As Long, ByVal inHigh As Long)

  'varray()  The varray to sort
  'inLow     First element of varray to start sort
  'inHigh    Last element of varray to start sort

  
   Dim pivot As String
   
   Dim tmpSwap As Long
   Dim tmpLow As Long
   Dim tmpHigh As Long
   Dim Qindex As Long
   
On Error GoTo Failed

   tmpLow = inLow
   tmpHigh = inHigh
   Qindex = inLow + inHigh
   Qindex = Qindex \ 2
   pivot = varray(Qindex)

   While (tmpLow <= tmpHigh)
   
    If CloseClick Then Exit Sub
   
        DisplayLines tmpLow, tmpHigh
   
      While (varray(tmpLow) < pivot And tmpLow < inHigh)
        DisplayLines tmpLow, tmpHigh
        Compares = Compares + 1
        ShowSwaps
         tmpLow = tmpLow + 1
        DisplayLines tmpLow, Qindex
      Wend
        Compares = Compares + 1
        ShowSwaps
   
   
      While (pivot < varray(tmpHigh) And tmpHigh > inLow)
        DisplayLines tmpHigh, Qindex
        Compares = Compares + 1
        ShowSwaps
         tmpHigh = tmpHigh - 1
      Wend
        Compares = Compares + 1
        ShowSwaps


      If (tmpLow <= tmpHigh) Then
        
      If (tmpLow < tmpHigh) Then
        Swaps = Swaps + 1
      End If
        ShowSwaps
         tmpSwap = varray(tmpLow)
         varray(tmpLow) = varray(tmpHigh)
         varray(tmpHigh) = tmpSwap
         tmpLow = tmpLow + 1
         tmpHigh = tmpHigh - 1

      End If

   Wend
   
   If (inLow < tmpHigh) Then QuickSort varray, inLow, tmpHigh
   If (tmpLow < inHigh) Then QuickSort varray, tmpLow, inHigh
   
   
    ShowSwaps
   

   On Error GoTo 0
   Exit Sub

Failed:

    MsgBox "Error " & Err.Number & " (" & Err.Description & ") in procedure QuickSort of Module modSort"

End Sub


'   The SelectionSort iterates through the list carrying the lowest
'   value from the first comparison down the list until it finds
'   a place that has a lesser value and inserting it there. Then
'   it rewinds and starts again. It does this until there are no more
'   swaps.


Public Sub SelectionSortNumbers(varray As Variant)

   Dim cnt1 As Long
   Dim cnt2 As Long
   Dim nMin As Long
   Dim tmp As Long
   Dim counter As Long
   

For cnt1 = LBound(varray) To UBound(varray) - 1
    If CloseClick Then Exit Sub

      nMin = cnt1

    For cnt2 = (cnt1 + 1) To UBound(varray)
        DisplayLines cnt2, nMin
        Compares = Compares + 1
        ShowSwaps
        If varray(cnt2) < varray(nMin) Then
            nMin = cnt2
        End If

    Next cnt2

      tmp = varray(nMin)
      varray(nMin) = varray(cnt1)
      varray(cnt1) = tmp
      Swaps = Swaps + 1
      ShowSwaps

Next cnt1

    ShowSwaps


End Sub


'   This is an optimized ShellSort, whereby on the final pass
'   if a swap is made, it backs up and rechecks for proper order
'   before continuing the iteration.



Public Sub ShellSort(varray As Variant)

   Dim cnt As Long
   Dim tmp As Long
   Dim nHold As Long
   Dim nHValue As Long
   Dim counter As Long
   
   
    nHValue = LBound(varray)
   
    Do
        nHValue = 3 * nHValue + 1
    Loop Until nHValue > UBound(varray)
   
Do
      nHValue = nHValue / 3

    For cnt = nHValue + LBound(varray) To UBound(varray)
        If CloseClick Then Exit Sub
        tmp = varray(cnt)
        nHold = cnt

        Do While varray(nHold - nHValue) > tmp
            Compares = Compares + 1
            Swaps = Swaps + 1
            DisplayLines cnt, nHold - nHValue
            varray(nHold) = varray(nHold - nHValue)
            nHold = nHold - nHValue
            If nHold < nHValue Then Exit Do
        Loop
        
        Compares = Compares + 1
        ShowSwaps
         varray(nHold) = tmp


    Next cnt

Loop Until nHValue = LBound(varray)

    ShowSwaps
    
End Sub




Private Sub DisplayLines(ByVal Y1 As Long, ByVal Y2 As Long)
    On Error Resume Next
    Main.DrawWidth = 3
    
    ClearLines
   ' top horizontal line
   Main.Line (Xleft, Main.Element(Y1).Top + Half)-(Xleft + Hlen, Main.Element(Y1).Top + Half), vbRed, BF
   ' bottom horizontal line
   Main.Line (Xleft, Main.Element(Y2).Top + Half)-(Xleft + Hlen, Main.Element(Y2).Top + Half), vbRed, BF
   ' vertical line
   Main.Line (Xleft + Hlen, Main.Element(Y1).Top + Half)-(Xleft + Hlen, Main.Element(Y2).Top + Half), vbRed, BF
       
   Main.Delay 5


    ShowSwaps
End Sub






Private Sub ShowSwaps()
    On Error Resume Next
    Main.ShowElements
    Main.NumSwaps = "Total Compares: " & Str(Compares) & "    Total Swaps: " & Str(Swaps)
End Sub





Private Sub ClearLines()
    On Error Resume Next
    Dim bcolor As Long
    bcolor = &H8000000F
    Main.Line (Xleft, Main.Element(0).Top)-(Xleft + Hlen, Main.Element(UBound(SortArray)).Top + 100), bcolor, BF
End Sub

